<?php

namespace Drupal\commerce_promo_link\Controller;

use Drupal\commerce_store\CurrentStoreInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides route responses for commerce promo link module.
 */
class PromoLink extends ControllerBase {

  use StringTranslationTrait;

  /**
   * The current store.
   *
   * @var \Drupal\commerce_store\CurrentStoreInterface
   */
  protected $currentStore;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The tempstore object.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStore
   */
  protected $tempStore;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The session.
   *
   * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
   */
  protected $session;

  /**
   * Container.
   *
   * @var \Symfony\Component\DependencyInjection\ContainerInterface
   */
  protected $container;

  /**
   * PromoLink constructor.
   *
   * @param \Drupal\commerce_store\CurrentStoreInterface $current_store
   * @param \Drupal\Core\Session\AccountInterface $current_user
   */
  public function __construct(ContainerInterface $container, CurrentStoreInterface $current_store, AccountInterface $current_user, PrivateTempStoreFactory $temp_store_factory, MessengerInterface $messenger) {
    $this->container = $container;
    $this->currentStore = $current_store->getStore();
    $this->currentUser = $current_user;
    $this->entityTypeManager = $this->entityTypeManager();
    $this->tempStore = $temp_store_factory->get('commerce_promo_link');
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container,
      $container->get('commerce_store.current_store'),
      $container->get('current_user'),
      $container->get('tempstore.private'),
      $container->get('messenger')
    );
  }

  /**
   * @param $code
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function addPromoCode($code) {

    // Force session start if we don't already have a session.
    $session = $this->container->get('session');
    if (!$session->isStarted()) {
      $session->migrate();
    }

    $code = Html::escape($code);
    $this->tempStore->set('coupon_code', $code);
    $this->messenger->addMessage($this->t('Your coupon code @code will be automatically applied during the checkout.', ['@code' => $code]));
    return $this->redirect('<front>');
  }

}
